/* dhf-test.c - test module for the dyn-host-filter
           Copyright (C) 2006, 2007 Simeon Simeonov

 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2, or (at your option)
 any later version.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software Foundation,
 Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA. */


#include <unistd.h>
#include <stdlib.h>

#include "parser.h"


#define LINE_BUFFER_SIZE 1024

static void print_help(void);
static void print_rules(void);
static void print_defaults(void);
static int check_host_entry(const char *str);
static int check_ip_entry(const char *str);
static LinkedList *rule_list;
static DHFConf *dhf_config;

#define REPLY_CODE_STR(a) (a ? DHFSTR_451 : DHFSTR_553)
#define DEFAULT_ACTION_STR(a) (a ? DHFSTR_PASS : DHFSTR_BLOCK)


#define PROGRAM_NAME "dhf-test"
#define PROGRAM_VERSION "1.0"

#define IP_REGEXP "^[0-9]{1,3}\\.[0-9]{1,3}\\.[0-9]{1,3}\\.[0-9]{1,3}$"    /* this regexp will determine whether the entry is IP or not */ 

char *program_name;

int 
main(int argc, char **argv)
{

  unsigned int i, file_arg, test_arg, list_arg, defaults_arg; /* file_arg points to the file index in argv or 0 if -f is not used... */
  char *entry;
  regex_t compiled;
  
  dhf_config = (DHFConf *) malloc(sizeof(DHFConf));

  rule_list = (LinkedList *) malloc(sizeof(LinkedList));
  list_init(rule_list);

  /* assigning the default values before parsing */
  dhf_config->default_action = DHF_ACTION_PASS;
  dhf_config->default_unresolved_action = DHF_ACTION_BLOCK;
  dhf_config->default_ip_action = DHF_ACTION_BLOCK;

  dhf_config->block_host_code = DHF_SMTP_553;
  dhf_config->block_unresolved_code = DHF_SMTP_451;
  dhf_config->block_ip_code = DHF_SMTP_451;

  file_arg = test_arg = list_arg = defaults_arg = 0;
  program_name = argv[0];
  /* ------------------------------------------- */

  if (argc < 2) {
    print_help();
    exit(EXIT_FAILURE);
  }

  for (i = 0; i < argc; i++) {

    if ((strcmp (argv[i], "-h") == 0) || (strcmp (argv[i], "--help") == 0)) {
      print_help();
      exit(EXIT_SUCCESS);
    }

    else if ((strcmp (argv[i], "-v") == 0) || (strcmp (argv[i], "--version") == 0)) {
      printf("%s v%s\n", PROGRAM_NAME, PROGRAM_VERSION);
      exit(EXIT_SUCCESS);
    }

    else if ((strcmp (argv[i], "-f") == 0) || (strcmp (argv[i], "--file") == 0))
      file_arg = i + 1;

    else if ((strcmp (argv[i], "-l") == 0) || (strcmp (argv[i], "--list") == 0))
      list_arg = 1;

    else if ((strcmp (argv[i], "-t") == 0) || (strcmp (argv[i], "--test") == 0))
      test_arg = i + 1;

    else if ((strcmp (argv[i], "-d") == 0) || (strcmp (argv[i], "--list-defaults") == 0))
      defaults_arg = 1;

    else if ((strcmp (argv[i], "-a") == 0) || (strcmp (argv[i], "--list-all") == 0))
      defaults_arg = list_arg = 1;

  }


  /* parse the config file */
  if ((file_arg > 1) && (file_arg < argc)) {
    if (parse_dhf_test_rules(argv[file_arg], dhf_config, rule_list) != 0)
      exit(EXIT_FAILURE);
  } else {
    if (parse_dhf_test_rules(DHF_CONF_FILE, dhf_config, rule_list) != 0) /* use the default config file */
      exit(EXIT_FAILURE);
  }


  if (defaults_arg)
    print_defaults();


  if (list_arg)
    print_rules();


  if ((test_arg > 1) && (test_arg < argc))
    entry = argv[test_arg];
  else 
    exit(EXIT_SUCCESS);
  

  /* now we check whether the entry is IP in order to decide which default status/reply code to use */
  regcomp (&compiled, IP_REGEXP, REG_EXTENDED);
  if (regexec(&compiled, entry, 0, NULL, 0) == 0) {

    if (dhf_config->default_unresolved_action) { 

      if (check_ip_entry(entry)) {
        printf("%s -> %s\n",entry, DHFSTR_PASSED);
      } else {
        printf("%s -> %s (%s)\n",entry, DHFSTR_BLOCKED, REPLY_CODE_STR(dhf_config->block_ip_code));
      }

    } else { 

      printf("%s -> %s (%s) (%s)\n",entry, DHFSTR_BLOCKED, DHFSTR_UNRESOLVED, REPLY_CODE_STR(dhf_config->block_unresolved_code));

    } 

    

  } else {

    if (check_host_entry(entry)) {
      printf("%s -> %s\n",entry, DHFSTR_PASSED);
    } else {
      printf("%s -> %s (%s)\n",entry, DHFSTR_BLOCKED, REPLY_CODE_STR(dhf_config->block_host_code));
    }

  }  

  exit(EXIT_SUCCESS);

}


static int
check_host_entry(const char *str)
{

  DHFRule *tmp_rule;
  tmp_rule = rule_list->head;

  while (tmp_rule != NULL) {

    if (regexec(&tmp_rule->regexp, str, 0, NULL, 0) == 0)
      return tmp_rule->action;

    tmp_rule = tmp_rule->next;

  }

  return dhf_config->default_action;

}


static int
check_ip_entry(const char *str)
{

  DHFRule *tmp_rule;
  tmp_rule = rule_list->head;

  while (tmp_rule != NULL) {

    if (regexec(&tmp_rule->regexp, str, 0, NULL, 0) == 0)
      return tmp_rule->action;

    tmp_rule = tmp_rule->next;

  }

  return dhf_config->default_ip_action;

}


static void 
print_help(void)
{

  printf("\n%s v%s - Simeon Simeonov\n\n", PROGRAM_NAME, PROGRAM_VERSION);
  printf("%s: %s [ -a | -d | -l ] [ -f < cfg-file > ] [ -t < entry > ] | -h | -v ]\n\n", DHFSTR_HELP_USAGE, program_name);
  printf("%s: \n", DHFSTR_HELP_OPTIONS);
  printf("\t-a \t\t - %s\n", DHFSTR_HELP_A);
  printf("\t-d \t\t - %s\n", DHFSTR_HELP_D);
  printf("\t-l \t\t - %s\n", DHFSTR_HELP_L);
  printf("\t-f < cfg-file >  - %s\n", DHFSTR_HELP_F);
  printf("\t-t < entry > \t - %s\n", DHFSTR_HELP_T);
  printf("\t-h \t\t - %s\n", DHFSTR_HELP_H);
  printf("\t-v \t\t - %s\n\n", DHFSTR_HELP_V);

}


static void
print_rules(void)
{

  DHFRule *tmp_rule;

  if (rule_list->size < 1) {

    printf("%s\n", DHFSTR_NORULES);

  } else {

    tmp_rule = rule_list->head;

    while (tmp_rule != NULL) {

      if (tmp_rule->action == DHF_ACTION_BLOCK) {
        printf("%s -> ", DHFSTR_BLOCK);
      } else {
        printf("%s -> ", DHFSTR_PASS);
      }

      printf("%s\n", tmp_rule->rule);

      tmp_rule = tmp_rule->next;

    }

  }

}


static void
print_defaults(void)
{

  printf("default_action -> %s\n", DEFAULT_ACTION_STR(dhf_config->default_action));
  printf("default_unresolved_action -> %s\n", DEFAULT_ACTION_STR(dhf_config->default_unresolved_action));
  printf("default_ip_action -> %s\n", DEFAULT_ACTION_STR(dhf_config->default_ip_action));

  printf("host_code -> %s\n", REPLY_CODE_STR(dhf_config->block_host_code));
  printf("unresolved_code -> %s\n", REPLY_CODE_STR(dhf_config->block_unresolved_code));
  printf("ip_code -> %s\n", REPLY_CODE_STR(dhf_config->block_ip_code));

}
