/* dyn-host-filter.c - IP/Hostname based filter for qmail
           Copyright (C) 2006, 2007 Simeon Simeonov

 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2, or (at your option)
 any later version.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software Foundation,
 Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA. */


#include <unistd.h>
#include <stdlib.h>

#include "parser.h"

#ifdef ENABLE_LOGGING
#include <time.h>
#endif



#define LINE_BUFFER_SIZE 1024

static int check_host_entry(const char *str);
static int check_ip_entry(const char *str);

static LinkedList *rule_list;
static DHFConf *dhf_config;

/* update the RBLSMTPD env. variable in order to achieve SMTP-code 451 or 553 */
#define DHF_REJECT_ENTRY(a) (a ? putenv(ENV_SMTP_REJECT_451) : putenv(ENV_SMTP_REJECT_553))


int main(int argc, char **argv)
{

  char *entry, *remote_ip;
  
#ifdef ENABLE_LOGGING
  char tbuffer[LINE_BUFFER_SIZE];
  time_t curtime;
  struct tm *loctime;
  
  FILE *log_file;
  
  curtime = time (NULL);
  loctime = localtime (&curtime);
  strftime (tbuffer, LINE_BUFFER_SIZE, "%d:%m:%Y %H:%M:%S", loctime);
#endif

  if (argc < 2) {
    fputs("usage: dyn-host-filter program [arguments]\n", stderr);
    return 111;
  }


#ifdef ENABLE_LOGGING
  if ((log_file = fopen(DHF_LOG_FILE, "a")) == NULL) {
    return 111;
  }
#endif


  rule_list = (LinkedList *) malloc(sizeof(LinkedList));
  list_init(rule_list);

  dhf_config = (DHFConf *) malloc(sizeof(DHFConf));

  /* assigning the default values before parsing */
  dhf_config->default_action = DHF_ACTION_PASS;
  dhf_config->default_unresolved_action = DHF_ACTION_BLOCK;
  dhf_config->default_ip_action = DHF_ACTION_BLOCK;

  dhf_config->block_host_code = DHF_SMTP_553;
  dhf_config->block_unresolved_code = DHF_SMTP_451;
  dhf_config->block_ip_code = DHF_SMTP_451;
  /* ------------------------------------------- */

  if (parse_dhf_rules(dhf_config, rule_list) != 0) {
    return 111;
  }

  entry = getenv("TCPREMOTEHOST"); /* we fetch the hostname env. variable */

  if (entry == NULL) {

    remote_ip = getenv("TCPREMOTEIP");

    if (dhf_config->default_unresolved_action == DHF_ACTION_BLOCK) {

      DHF_REJECT_ENTRY(dhf_config->block_unresolved_code);
#ifdef ENABLE_LOGGING
      fprintf(log_file, "%s -> %s %s -> %s\n", tbuffer, DHFSTR_ERR_RESOLV, remote_ip, DHFSTR_BLOCKED);
#endif
      
    } else {

      if (check_ip_entry(remote_ip)) {
#ifdef ENABLE_LOGGING
        fprintf(log_file, "%s -> %s -> %s\n", tbuffer, remote_ip, DHFSTR_PASSED);
#endif
      } else {
        DHF_REJECT_ENTRY(dhf_config->block_ip_code);
#ifdef ENABLE_LOGGING
        fprintf(log_file, "%s -> %s -> %s\n", tbuffer, remote_ip, DHFSTR_BLOCKED);
#endif      
      }


    } 

  } else if (entry == 0) {
    
#ifdef ENABLE_LOGGING
    fprintf(log_file, "%s -> TCPREMOTEHOST = 0\n", tbuffer);
#endif  

  } else {

#ifdef ENABLE_LOGGING
    fprintf(log_file, "%s -> TCPREMOTEHOST = %s", tbuffer, entry);
#endif
    if (check_host_entry(entry)) {
#ifdef ENABLE_LOGGING
      fprintf(log_file, " -> %s\n", DHFSTR_PASSED);
#endif
    } else {
      DHF_REJECT_ENTRY(dhf_config->block_host_code);
#ifdef ENABLE_LOGGING
      fprintf(log_file, " -> %s\n", DHFSTR_BLOCKED);
#endif
    }

  }

#ifdef ENABLE_LOGGING
  fclose(log_file);
#endif

  execvp(argv[1], argv+1);
  fputs("error: execution of dyn-host-filter failed\n", stderr);
  return 111;

}


static int
check_host_entry(const char *str)
{

  DHFRule *tmp_rule;
  tmp_rule = rule_list->head;

  while (tmp_rule != NULL) {

    if (regexec(&tmp_rule->regexp, str, 0, NULL, 0) == 0)
      return tmp_rule->action;

    tmp_rule = tmp_rule->next;

  }

  return dhf_config->default_action;

}


static int
check_ip_entry(const char *str)
{

  DHFRule *tmp_rule;
  tmp_rule = rule_list->head;

  while (tmp_rule != NULL) {

    if (regexec(&tmp_rule->regexp, str, 0, NULL, 0) == 0)
      return tmp_rule->action;

    tmp_rule = tmp_rule->next;

  }

  return dhf_config->default_ip_action;

}
